import os
import pathlib
from textwrap import dedent

from invoke import Program as Programme

programme = Programme()


def main():
    if not os.path.exists("tasks.py"):
        print("Not tasks.py found; creating default...")
        name = pathlib.Path.cwd().name
        with open("tasks.py", "w") as f:
            f.write(
                dedent(
                    f"""\
                import hedy


                namespace = hedy.app("{name}")
                """
                )
            )
    programme.run()



import setuptools
from invoke import Collection, task

from . import tasks

__ALL__ = ["app", "setup"]
TASKS = Collection("tasks")


def add_task(t, **project_args):
    @task(
        name=t.name,
        optional=t.optional,
    )
    def wrapped_task(c, **task_args):
        return t(c, **project_args, **task_args)

    wrapped_task.__doc__ = t.__doc__
    TASKS.add_task(wrapped_task, name=t.__name__)


def app(name):
    add_task(tasks.run, name=name)
    add_task(tasks.deb, name=name)
    add_task(tasks.stage)
    return TASKS


def setup(name, github_owner, **kwargs):
    sensible_defaults = dict(
        name=name,
        use_scm_version={
            "local_scheme": "dirty-tag",
            "write_to": f"{name}/_version.py",
            "fallback_version": "0.0.0",
        },
        long_description="# name",
        long_description_content_type="text/markdown",
        url=f"https://github.com/{github_owner}/{name}",
        packages=[name],
        include_package_data=True,
        classifiers=[
            # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
            "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
            "Intended Audience :: Developers",
            "Operating System :: Unix",
            "Operating System :: POSIX",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.5",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: Implementation :: CPython",
            "Topic :: Internet :: WWW/HTTP",
        ],
        project_urls={
            "Issue Tracker": f"https://github.com/{github_owner}/{name}/issues",
        },
        setup_requires=[
            "pytest-runner",
            "setuptools_scm>=3.3.1",
            "pre-commit",
        ],
    )

    params = {**sensible_defaults, **kwargs}
    setuptools.setup(**params)



import os
import pathlib
from textwrap import dedent

from invoke import task

DOCKERFILE = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "Dockerfile.debian"
)


DOCKERFILE_STAGE = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "Dockerfile.stage"
)


@task
def run(c, name):
    if not os.path.exists("manage.py"):
        print("No manage.py found; creating default...")
        with open("manage.py", "w") as f:
            f.write(
                dedent(
                    """
            #!/usr/bin/env python


            import sys


            if __name__ == '__main__':
                from django.core.management import execute_from_command_line

                execute_from_command_line(sys.argv)
            """
                )
            )

    package_path = pathlib.Path.cwd() / name

    if not package_path.exists():
        package_path.mkdir()

    settings_path = package_path / "settings.py"
    if not settings_path.exists():
        print("No settings.py found; creating default...")
        with open(settings_path, "w") as f:
            f.write(
                dedent(
                    """
            import django12factor
            d12f = django12factor.factorise()

            DEBUG = d12f['DEBUG']
            """
                )
            )

    c.run(
        f"DJANGO_SETTINGS_MODULE={name}.settings DEBUG=true python manage.py runserver"
    )


@task
def deb(c, name):
    c.run(f"docker build -t hedy-debian-fpm -f {DOCKERFILE} .")
    fpm = "docker run -v $PWD:/app -w /app hedy-debian-fpm"
    version = c.run("python setup.py --version").stdout.strip()
    c.run("rm -rf dist")
    c.run("python setup.py sdist")
    c.run(
        f"{fpm} --verbose --input-type virtualenv --output-type deb "
        f"--name virtualenv-{name} --version {version} --architecture all "
        f"--depends python3-distutils --python-bin /usr/bin/python3 "
        f"--prefix /opt/{name} dist/*"
    )
    c.run(
        f"{fpm} --verbose --input-type dir --output-type deb "
        f"--name {name} --version {version} --depends virtualenv-{name} --architecture all "
        f"--depends apache2 --depends libapache2-mod-wsgi-py3 "
        f"--after-install deb/after-install --config-files /etc deb/etc=/"
    )


@task
def stage(c):
    c.run(f"docker build -t hedy-debian-stage -f {DOCKERFILE_STAGE} .")
    c.run("docker run -it -p 80:80 hedy-debian-stage", pty=True)
